//+------------------------------------------------------------------+
//|                                      NeutralHedge Overlay_v4.mq4 |
//|                                             2008.07.08 SwingMan |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright " 2008.07.08 SwingMan"
#property link      ""
// original idea:
//+------------------------------------------------------------------+
//|                                       OverLay Chart.mq4 Ver.1.10 |
//|        Copyright 2006-2007 S.B.T. | http://sufx.core.t3-ism.net |
//+------------------------------------------------------------------+
// - SwingMan / 2008.07.04
// v1 - simplify the formulas
//    - take begin/end of the calculation period
// 2008.07.05  - some procedures moved in NH_Library
// v2 - moving window for overlay
// v4 - Mirroring parameter: -1,0,1 / 2008.07.07
//+------------------------------------------------------------------+

#include <NeutralHedge_Library.mqh>

#property indicator_chart_window

#property indicator_buffers 4
#property indicator_color1 DeepSkyBlue
#property indicator_color2 Orange
#property indicator_color3 Orange
#property indicator_color4 DeepSkyBlue


//---- inputs --------------------------------------------------------
extern string SubSymbol = "";
extern string Last_OverlayTime = "";
//extern string Last_OverlayTime = "2008.07.02 20:00";
extern string ____Mirroring____ = "1= mirroring, 0=autom, -1= not mirroring";
extern int Pair_Mirroring = 0;
extern int Period_Range = 72;
extern double Factor_Threshold = 0;
//--------------------------------------------------------------------

int Offset_Ratio;
datetime Last_Time;
bool Mirroring = true;

//---- constants
string sWinName1 = "NeutralHedge Overlay_v4", sWinName;
string sObject = "NHOv4_";
int fontSize_Text = 8;

//Indicator Buffers
double SubPrice[], SubPriceCont[];
double HighLine[], LowLine[];
double RangeUpTemp[], RangeDnTemp[];

//-- variables
double dPoint;
int iWindow;
int firstBar, lastBar;
datetime firstTime, lastTime, thisTime, oldTime;
double Close_Main, Close_Sub;
double HedgeRange, threshold;
bool Trade_EntryTimeOK;
string MainSymbol = "";

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{  
   if (MainSymbol == "") MainSymbol = Symbol();
   
   Set_DefaultPairMirroring(MainSymbol, Pair_Mirroring, SubSymbol, Mirroring);
   if (SubSymbol == "") {Alert("SubSymbol is missing!"); return;}
  
   dPoint     = MarketInfo(MainSymbol, MODE_POINT);
   
   //-- check time input   
   if (Last_OverlayTime == "") {
      Offset_Ratio = 0;
      Last_Time = iTime(MainSymbol, 0, 0);
      Trade_EntryTimeOK = true;
   } else {
      Last_Time = StrToTime(Last_OverlayTime);      
      if (Last_Time > iTime(MainSymbol, 0, 0)) {
         Trade_EntryTimeOK = false;
         string st = MainSymbol + " input ERROR: false Last_OverlayTime (" +Last_OverlayTime+")";
         Alert(st);   
      } else
         Trade_EntryTimeOK = true;
      if (Trade_EntryTimeOK == true)
         Offset_Ratio = iBarShift(MainSymbol, 0, Last_Time);
   }
   
   firstTime = iTime(MainSymbol, 0, Offset_Ratio + Period_Range);
   lastTime  = iTime(MainSymbol, 0, Offset_Ratio);  
     
   firstBar = iBarShift(MainSymbol, 0, firstTime);
   lastBar  = iBarShift(MainSymbol, 0, lastTime);
     
   sWinName = sWinName1 + " (" + MainSymbol + "-" + SubSymbol + ") ";
   IndicatorShortName(sWinName);   
   IndicatorBuffers(6);
   
   SetIndexBuffer( 0, SubPrice );
   SetIndexBuffer( 1, HighLine );
   SetIndexBuffer( 2, LowLine );
   SetIndexBuffer( 3, SubPriceCont );

   SetIndexBuffer( 4, RangeUpTemp);
   SetIndexBuffer( 5, RangeDnTemp);

   SetIndexStyle( 0, DRAW_LINE, STYLE_SOLID, 1);
   SetIndexStyle( 1, DRAW_LINE, STYLE_SOLID, 1);
   SetIndexStyle( 2, DRAW_LINE, STYLE_SOLID, 1);
   SetIndexStyle( 3, DRAW_LINE, STYLE_DOT, 1);
      
   SetIndexEmptyValue( 0, 0.0 );
   SetIndexEmptyValue( 1, 0.0 );
   SetIndexEmptyValue( 2, 0.0 );
   SetIndexEmptyValue( 3, 0.0 );
   SetIndexEmptyValue( 4, 0.0 );
   SetIndexEmptyValue( 5, 0.0 );

   SetIndexLabel( 0, SubSymbol + " ranged" );
   SetIndexLabel( 1, NULL );
   SetIndexLabel( 2, NULL );
   SetIndexLabel( 3, SubSymbol + " contin" );
         
   IndicatorDigits(Digits);
     
   DeleteOwnObjects(sObject);
   iWindow = WindowFind(sWinName);
   
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   DeleteOwnObjects(sObject);
   Comment("");
   return(0);
}


//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   if (SubSymbol == "") return(-1);
   
   double CurRangeHigh, CurRangeLow, CurRangeCenter;
   double SubRangeHigh, SubRangeLow, SubRangeCenter, SubClose;
   double PipsRatio;
   
   //-- Calculate Chart Ratio
   CurRangeHigh  = iHigh( MainSymbol, 0, iHighest(MainSymbol, 0, MODE_HIGH, Period_Range, lastBar));
   CurRangeLow   = iLow (MainSymbol,  0, iLowest(MainSymbol,  0, MODE_LOW,  Period_Range, lastBar));
   CurRangeCenter= ( CurRangeHigh + CurRangeLow ) * 0.5;
      
   if ( Mirroring == true ) {
      SubRangeHigh = iLow(  SubSymbol, 0, iLowest( SubSymbol, 0, MODE_LOW, Period_Range, lastBar ) );
      SubRangeLow  = iHigh( SubSymbol, 0, iHighest(SubSymbol, 0, MODE_HIGH,Period_Range, lastBar ) );
   } else {
      SubRangeHigh = iHigh( SubSymbol, 0, iHighest(SubSymbol, 0, MODE_HIGH,Period_Range, lastBar ) );
      SubRangeLow  = iLow(  SubSymbol, 0, iLowest( SubSymbol, 0, MODE_LOW, Period_Range, lastBar ) );
   }

   SubRangeCenter = ( SubRangeHigh + SubRangeLow ) * 0.5;
   PipsRatio = ( CurRangeHigh - CurRangeLow )  / ( SubRangeHigh - SubRangeLow );
   
   // Draw ratio   
   for (int iBar = lastBar; iBar < firstBar; iBar++) {
      SubClose = iClose( SubSymbol, 0, iBar ) - SubRangeCenter;
      Close_Sub = CurRangeCenter + SubClose * PipsRatio;      
   
      SubPrice[iBar] = Close_Sub;
      HighLine[iBar] = CurRangeHigh;
      LowLine[iBar] = CurRangeLow;
      
      //-- ranges
      Close_Main = iClose(MainSymbol, 0, iBar);
      double range = (Close_Main - Close_Sub) / dPoint;      
      
      if (range > 0) 
         RangeUpTemp[iBar] = range; // positive ranges
      else 
         RangeDnTemp[iBar] = range; // negative ranges            
   }
   //-- continuation of the Sub Prices
//Print("lastBar=",lastBar);   
   if (lastBar > 0) {
      for (iBar = 0; iBar <= lastBar; iBar++) {
         SubClose = iClose( SubSymbol, 0, iBar ) - SubRangeCenter;
         Close_Sub = CurRangeCenter + SubClose * PipsRatio;      
   
         SubPriceCont[iBar] = Close_Sub;
      }
   }
  
   //-- HedgeRange   
   Close_Main = iClose(MainSymbol, 0, lastBar);
   SubClose = iClose( SubSymbol, 0, lastBar ) - SubRangeCenter;
   Close_Sub = CurRangeCenter + SubClose * PipsRatio;   
   
   HedgeRange = (Close_Main - Close_Sub) / dPoint;
   
   //-- threshold
   threshold = Get_ThresholdValue(Factor_Threshold, RangeUpTemp, RangeDnTemp, 
                      lastBar, firstBar, 1, 0);
   //-- comment
   if (Trade_EntryTimeOK == true) 
      Show_Comment();   
//----
   return(0);
}
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//    Show Comment
//+------------------------------------------------------------------+
void Show_Comment()
{  
   string st, sTrading, sThreshold, sTradingOK; 
   int Digits_Sub = MarketInfo(SubSymbol, MODE_DIGITS);

   //-- SubSymbol close   
   double SubClose = iClose( SubSymbol, 0, 0 );   
   string sSubClose = SubSymbol + " C= " + DoubleToStr(SubClose,Digits_Sub);
   
   //if (Last_OverlayTime != "") string sLast_OverlayTime = Last_OverlayTime;
   //else sLast_OverlayTime = TimeToStr(iTime(MainSymbol,0,0));
   
   //-- results
   string line = "==================\n";
   string firstDate = TimeToStr(firstTime);
   string lastDate  = TimeToStr(lastTime);
   string sHedgeRange = DoubleToStr(HedgeRange, 1);
   
   string sClose_Main = "last ranged C= " + MainSymbol + " " + DoubleToStr(Close_Main, Digits);
   string sClose_Sub  = "last ranged C= " + SubSymbol+ " " + DoubleToStr(Close_Sub, Digits);
   
   string sPeriodRatio = DoubleToStr(Period_Range,0);
   if (HedgeRange < 0) {
      sTrading = "buy "+MainSymbol + ", sell " + SubSymbol;
      sThreshold = DoubleToStr(-threshold,1);
      if (HedgeRange < -threshold) sTradingOK = "OK"; else sTradingOK = "NONE";
   } else {
      sTrading = "sell "+MainSymbol + ", buy " + SubSymbol;
      sThreshold = DoubleToStr(threshold,1);
      if (HedgeRange > threshold) sTradingOK = "OK"; else sTradingOK = "NONE";
   }   
   if (Mirroring == true) string sMirroring = "Mirroring= true"; else sMirroring = "Mirroring= false";
   
   //-- comment string ...............................................
   st = sSubClose + "\n";
   st = st + sWinName + "\n";
   st = st + line;   
   st = st + sMirroring + "\n";
   st = st + "firstBar= " + firstDate + "\n";
   st = st + "lastBar= " + lastDate + "\n";
   //st = st + "OverlayTime= " + sLast_OverlayTime + "\n";
   st = st + "PeriodRange= " + sPeriodRatio + "\n";
   st = st + sClose_Main + "\n";  
   st = st + sClose_Sub + "\n";        
   st = st + "HedgeRange= " + sHedgeRange + "\n";
   st = st + "Threshold= " + sThreshold + "\n";
   st = st + "Trading: " + sTradingOK + "\n";
   st = st + sTrading + "\n";
   
   st = st + line;
   Comment(st);
}

