//+------------------------------------------------------------------+
//|                                        NeutralHedge osc_v4.4.mq4 |
//|                                             2008.07.08 SwingMan |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright " 2008.07.08 SwingMan"
#property link      ""
// source code:
//+------------------------------------------------------------------+
//|                                       OverLay Chart.mq4 Ver.1.10 |
//|                                      Copyright 2006-2007 S.B.T. |
//|                                     http://sufx.core.t3-ism.net/ |
//+------------------- DO NOT REMOVE THIS HEADER --------------------+
//|  This script is free to use/distribute/modify and re-distribute. |
//|                                  (Limited to noncommercial use.) |
//+------------------------------------------------------------------+
// changes:
//+------------------------------------------------------------------+
// - fewhills  / 2008.07.02
// I have modified the indicator with 2 extra parameters: starting time and TF period.
// The time parameter is the usual MT4 method: yyyy.mm.dd hh:mm
// The TF period is default at 1 Hr TF = 60 
//+------------------------------------------------------------------+
// - SwingMan / 2008.07.03
// v1 - draw histogram of ratios
// v2 - write trading results
// v3 - calculation of threshold with StdDev
//    - take automatic more as 40 bars
// v4 - take fixed XX bars for every ratio calculation
// 2008.07.05  - some procedures moved in NH_Library
// v4.2 - write results and comments like v3.2
// v4.4 - Mirroring parameter: -1,0,1 / 2008.07.08
//      - Results with spread
//+------------------------------------------------------------------+

#include <NeutralHedge_Library.mqh>

#property indicator_separate_window
#property indicator_buffers 4

#property indicator_color1 MediumSeaGreen
#property indicator_color2 DeepPink
#property indicator_color3 Magenta
#property indicator_color4 Aqua

#property indicator_level1 0
#property indicator_levelcolor Gray
#property indicator_levelstyle STYLE_SOLID

//---- inputs --------------------------------------------------------
extern string SubSymbol = "";
extern int Period_Range = 125;
extern int nDrawBars = 144;
extern string ____Mirroring____ = "1= mirroring, 0=autom, -1= not mirroring";
extern int Pair_Mirroring = 0;
extern string Trade_EntryTime = "";
//extern string Trade_EntryTime = "2008.07.02 20:00";
extern string _____Order____ = "BUY=1,  SELL=-1";
extern int Order_MainPair = 0;
extern double Factor_StdDevThreshold = 0;
//--------------------------------------------------------------------

//---- constants
string sWinName1 = "NeutralHedge osc_v4.4", sWinName;
string sObject = "NH44_";
int fontSize_Text = 8;
int iLabelCorner = 1;
int xx1 = 80;
int xx2 = 2;
int xx3 = 60;
int xx4 = 30;
int yy0 = 3, yy;
int yStep = 10;
double Factor_Threshold;

//Indicator Buffers
double RangeUp[], RangeDn[];
double RangeUpTemp[], RangeDnTemp[];
double thresholdUp[], thresholdDn[];

//-- variables
double dPoint;
int iWindow;
double HedgeRange, lastThreshold;
int threshold;
bool Trade_EntryTimeOK;
int lastBar = 0;
bool Mirroring = true;
string MainSymbol = "";

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{  
   Factor_Threshold = Factor_StdDevThreshold;
   
   if (MainSymbol == "") MainSymbol = Symbol();
   Set_DefaultPairMirroring(MainSymbol, Pair_Mirroring, SubSymbol, Mirroring);
   
   if (SubSymbol == "") {Alert("SubSymbol is missing!"); return;}
   if (nDrawBars == 0) nDrawBars = Period_Range;
   
   //-- check Trade_EntryTime input
   if (Trade_EntryTime != "") {
      datetime Entry_Time = StrToTime(Trade_EntryTime);
      if (Entry_Time > iTime(MainSymbol, 0, 0)) {
         Trade_EntryTimeOK = false;
         string st = MainSymbol + " input ERROR: false Trade_EntryTime (" +Trade_EntryTime+")";
         Alert(st);
      } else
         Trade_EntryTimeOK = true;
   }
   //-- check order setting
   if (Trade_EntryTimeOK == true) {
      if (Order_MainPair == 0) {
         st = MainSymbol +"  " + "Order_MainPair is missing! Enter BUY=1 or SELL=-1";
         Alert(st);
         Trade_EntryTimeOK = false;
      }
   }
   
   dPoint = MarketInfo(MainSymbol, MODE_POINT);   
      
   sWinName = sWinName1 + " (" + MainSymbol + "-" + SubSymbol + ", " + Period_Range+") ";
   IndicatorShortName(sWinName);   
   IndicatorBuffers(6);

   SetIndexBuffer( 0, RangeDn );
   SetIndexBuffer( 1, RangeUp );
   SetIndexBuffer( 2, thresholdUp );
   SetIndexBuffer( 3, thresholdDn );
   
   SetIndexBuffer( 4, RangeUpTemp );
   SetIndexBuffer( 5, RangeDnTemp );

   SetIndexStyle( 0, DRAW_HISTOGRAM, STYLE_SOLID, 2);
   SetIndexStyle( 1, DRAW_HISTOGRAM, STYLE_SOLID, 2);
   
   //-- lines
   SetIndexStyle( 2, DRAW_LINE, STYLE_SOLID, 1); 
   SetIndexStyle( 3, DRAW_LINE, STYLE_SOLID, 1);
   //-- arrows
   //SetIndexStyle( 2, DRAW_ARROW, EMPTY, 1); SetIndexArrow( 2, 159);
   //SetIndexStyle( 3, DRAW_ARROW, EMPTY, 1); SetIndexArrow( 3, 159);
   
   SetIndexEmptyValue( 0, 0.0 );
   SetIndexEmptyValue( 1, 0.0 );
   SetIndexEmptyValue( 2, 0.0 );
   SetIndexEmptyValue( 3, 0.0 );
   SetIndexEmptyValue( 4, 0.0 );
   SetIndexEmptyValue( 5, 0.0 );   

   SetIndexLabel( 0, "BUY -" +MainSymbol );
   SetIndexLabel( 1, "SELL-" +MainSymbol );
   SetIndexLabel( 2, "Thresh UP" );
   SetIndexLabel( 3, "Thresh DN" );
         
   IndicatorDigits(Digits);

   ArrayInitialize( RangeUp, 0.0 );
   ArrayInitialize( RangeDn, 0.0 );
   ArrayInitialize( RangeUpTemp, 0.0 );
   ArrayInitialize( RangeDnTemp, 0.0 );
   ArrayInitialize( thresholdUp, 0.0 );
   ArrayInitialize( thresholdDn, 0.0 );
      
   DeleteOwnObjects(sObject);
   
   iWindow = WindowFind(sWinName);
   
   return(0);
}


//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{  
   DeleteOwnObjects(sObject);
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
   if (SubSymbol == "") return(-1);
      
   double CurRangeHigh, CurRangeLow, CurRangeCenter, CurClose;
   double SubRangeHigh, SubRangeLow, SubRangeCenter, SubClose;
   double PipsRatio;
   int i, iBar;
         
   //-- Initialize Buffers
	RefreshRates();
	ArrayInitialize( RangeUpTemp, 0.0 );
   ArrayInitialize( RangeDnTemp, 0.0 );   
   
   for (iBar=0; iBar < nDrawBars; iBar++) {
      //-- Calculate Chart Ratio
      CurRangeHigh = High[iHighest(MainSymbol, 0, MODE_HIGH, Period_Range, iBar)];
      CurRangeLow  = Low[iLowest(MainSymbol, 0, MODE_LOW, Period_Range, iBar)];
      CurRangeCenter = ( CurRangeHigh + CurRangeLow ) * 0.5;
      
      if ( Mirroring == true ) {
         SubRangeHigh = iLow(  SubSymbol, 0, iLowest( SubSymbol, 0, MODE_LOW, Period_Range, iBar ) );
         SubRangeLow  = iHigh( SubSymbol, 0, iHighest(SubSymbol, 0, MODE_HIGH,Period_Range, iBar ) );
      } else {
         SubRangeHigh = iHigh( SubSymbol, 0, iHighest(SubSymbol, 0, MODE_HIGH,Period_Range, iBar ) );
         SubRangeLow  = iLow(  SubSymbol, 0, iLowest( SubSymbol, 0, MODE_LOW, Period_Range, iBar ) );
      }

      SubRangeCenter = ( SubRangeHigh + SubRangeLow ) * 0.5;
      PipsRatio = ( CurRangeHigh - CurRangeLow )  / ( SubRangeHigh - SubRangeLow );

      //-- Draw ratio
      SubClose = iClose( SubSymbol, 0, iBar ) - SubRangeCenter;      

      double Close_Sub = CurRangeCenter + SubClose * PipsRatio;     
      double Close_Main = iClose(MainSymbol, 0, iBar);
      double range = (Close_Main - Close_Sub) / dPoint;      
      
      if (range > 0) 
        RangeUp[iBar] = range; // positive ranges
      else 
        RangeDn[iBar] = range; // negative ranges
      //-- last range  
      if (iBar == 0) HedgeRange = range;
              
      //-- moving window with Period_Range bars           
      //-- treshold
     	for ( i = iBar; i < iBar + Period_Range; i ++ ) {
         SubClose = iClose( SubSymbol, 0, i ) - SubRangeCenter;      

         Close_Sub = CurRangeCenter + SubClose * PipsRatio;     
         Close_Main = iClose(MainSymbol, 0, i);
         range = (Close_Main - Close_Sub) / dPoint;      
      
         if (range > 0) 
            RangeUpTemp[i] = range; // positive ranges
         else 
            RangeDnTemp[i] = range; // negative ranges            
      }
      threshold = Get_ThresholdValue(Factor_Threshold, RangeUpTemp, RangeDnTemp, 
                      iBar, iBar+Period_Range, 1, 0);                          
      thresholdUp[iBar] = threshold;
      thresholdDn[iBar] = -threshold;
      
      //-- last threshold      
      if (iBar == 0) lastThreshold = threshold;
   }

   //-- write win/loss; comment
   if (Trade_EntryTimeOK) Write_WinLossTrading();
   Write_CommentTrading();
//----
   return(0);
}
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//    Write WinLoss Trading Result
//+------------------------------------------------------------------+
void Write_WinLossTrading()
{
   if (Trade_EntryTime == "") return;
   
   double WinLoss;
   string text1, text2, sEmpty = "EMPTY";
   
   int Digits_Sub = MarketInfo(SubSymbol,MODE_DIGITS);
   double dPoint_Sub = MarketInfo(SubSymbol,MODE_POINT);
   int MainSpread =  MarketInfo(MainSymbol,MODE_SPREAD);
   int SubSpread  =  MarketInfo(SubSymbol,MODE_SPREAD);

   string sumSpread = "(-Spread= " + DoubleToStr(MainSpread + SubSpread, 0) + ")";
      
   yy = yy0;
   iWindow = WindowFind(sWinName);
   iLabelCorner = 1;
      
   int MainEntryBar = iBarShift(MainSymbol,PERIOD_M1,StrToTime(Trade_EntryTime));
   int SubEntryBar = iBarShift(SubSymbol,PERIOD_M1,StrToTime(Trade_EntryTime));
   
   double Entry_Main = iClose(MainSymbol, PERIOD_M1, MainEntryBar);
   double Entry_Sub  = iClose(SubSymbol,  PERIOD_M1, SubEntryBar);
   
   double Price_Main = iClose(MainSymbol, PERIOD_M1, 0);
   double Price_Sub  = iClose(SubSymbol, PERIOD_M1, 0);
   
   //-- WinLoss results
   WinLoss = 0;   
   if (Order_MainPair == 1) {
      double Diff_Main = (Price_Main - Entry_Main) / dPoint;
      double Diff_Sub  = (Entry_Sub - Price_Sub) / dPoint_Sub;
      WinLoss = Diff_Main + Diff_Sub;
      string sOrder_MainPair = "buy " + MainSymbol + ": " + 
                           DoubleToStr(Entry_Main,Digits) + " / " + DoubleToStr(Price_Main,Digits) +
                           " = " + DoubleToStr(Diff_Main,0);
      string sOrder_Sub  = "sell " + SubSymbol + ": " + 
                           DoubleToStr(Entry_Sub,Digits_Sub) + " / " + DoubleToStr(Price_Sub,Digits_Sub) +
                           " = " + DoubleToStr(Diff_Sub,0);
   } else 
   if (Order_MainPair == -1) {
      Diff_Main = -(Price_Main - Entry_Main) / dPoint;
      Diff_Sub  = -(Entry_Sub - Price_Sub) / dPoint_Sub;
      WinLoss = Diff_Main + Diff_Sub;
      sOrder_MainPair = "sell " + MainSymbol + ": " + 
                           DoubleToStr(Entry_Main,Digits) + " / " + DoubleToStr(Price_Main,Digits) +
                           " = " + DoubleToStr(Diff_Main,0);
      sOrder_Sub  = "buy " + SubSymbol + ": " + 
                           DoubleToStr(Entry_Sub,Digits_Sub) + " / " + DoubleToStr(Price_Sub,Digits_Sub) +
                           " = " + DoubleToStr(Diff_Sub,0);
   } else {
      sOrder_MainPair = "trade " + MainSymbol + ": " + 
                           "NONE" + " / " + DoubleToStr(Price_Main,Digits);
      sOrder_Sub  = "trade " + SubSymbol + ": " + 
                           "NONE" + " / " + DoubleToStr(Price_Sub,Digits_Sub);
   }
   
   //-- maxWin results
   double maxWin = -EMPTY_VALUE; 
   double mPrice_Main, mPrice_Sub, mDiff_Main, mDiff_Sub, mWinLoss;
   int iBar;  
   if (Order_MainPair == 1) {
      for (iBar =0; iBar < MainEntryBar; iBar++) {
         mPrice_Main = iClose(MainSymbol, PERIOD_M1, iBar);
         mPrice_Sub  = iClose(SubSymbol, PERIOD_M1, iBar);
         mDiff_Main = (mPrice_Main - Entry_Main) / dPoint;
         mDiff_Sub  = (Entry_Sub - mPrice_Sub) / dPoint_Sub;
         mWinLoss = mDiff_Main + mDiff_Sub;
         if (mWinLoss > maxWin) maxWin = mWinLoss;
      }
   } else 
   if (Order_MainPair == -1) {
      for (iBar =0; iBar < MainEntryBar; iBar++) {
         mPrice_Main = iClose(MainSymbol, PERIOD_M1, iBar);
         mPrice_Sub  = iClose(SubSymbol, PERIOD_M1, iBar);
         mDiff_Main = -(mPrice_Main - Entry_Main) / dPoint;
         mDiff_Sub  = -(Entry_Sub - mPrice_Sub) / dPoint_Sub;
         mWinLoss = mDiff_Main + mDiff_Sub;
         if (mWinLoss > maxWin) maxWin = mWinLoss;
      }   
   }

   //-- write results
   //......................................................
   text1 = "Entry time:";
   text2 = Trade_EntryTime;
   Write_Label(iWindow, iLabelCorner, sObject, text1, text2, xx1+10, xx2, yy); yy = yy + yStep;

   //======================================================
   Write_Label(iWindow, iLabelCorner, sObject, sOrder_MainPair, "", xx2, xx2, yy); yy = yy + yStep;  
   Write_Label(iWindow, iLabelCorner, sObject, sOrder_Sub, "", xx2, xx2, yy); yy = yy + yStep;   
   
   //======================================================
   text1 = "WinLoss: " + DoubleToStr(WinLoss,0) + " (max= " + DoubleToStr(maxWin,0) + ")";
   Write_Label(iWindow, iLabelCorner, sObject, text1, "", xx2, xx2, yy); yy = yy + yStep;
   text1 = sumSpread;
   Write_Label(iWindow, iLabelCorner, sObject, text1, "", xx2, xx2, yy); yy = yy + yStep;
}


//+------------------------------------------------------------------+
//    Write Comment Trading
//+------------------------------------------------------------------+
void Write_CommentTrading()
{
   string sThreshold;
   int xx1 = 3;
   int yy = 12;
   iLabelCorner = 0;
   
   //...........   
   if (Mirroring ==  true) string sMirroring = "Mirroring= true"; else sMirroring = "Mirroring= false";
   Write_LabelComment(iWindow, iLabelCorner, sObject, sMirroring, "", xx1, xx1, yy);

   //...........
   yy = yy + yStep;
   string sPeriod_Range = "Period_Range= " + DoubleToStr(Period_Range,0);
   Write_LabelComment(iWindow, iLabelCorner, sObject, sPeriod_Range, "", xx1, xx1, yy);
   
   //...........
   yy = yy + yStep;
   string sHedgeRange = "HedgeRange= " + DoubleToStr(HedgeRange,1);
   Write_LabelComment(iWindow, iLabelCorner, sObject, sHedgeRange, "", xx1, xx1, yy);
   
   //...........
   yy = yy + yStep;
   if (HedgeRange < 0) sThreshold = "Threshold= " + DoubleToStr(-lastThreshold,1);
   else                sThreshold = "Threshold= " + DoubleToStr( lastThreshold,1);   
   Write_LabelComment(iWindow, iLabelCorner, sObject, sThreshold, "", xx1, xx1, yy);

   //...........
   yy = yy + yStep;   
   datetime LastTime = iTime(MainSymbol, 0, lastBar);
   string sLastTime = "(lastBar= " + TimeToStr(LastTime)+")";
   Write_LabelComment(iWindow, iLabelCorner, sObject, sLastTime, "", xx1, xx1, yy);
   
   //...........
   string sTrading, sTradingOK;
   if (HedgeRange < 0) {
      sTrading = "buy "+MainSymbol + ", sell " + SubSymbol;
      if (HedgeRange < -lastThreshold) sTradingOK = "Trading: OK"; else sTradingOK = "Trading: NONE";
   } else {
      sTrading = "sell "+MainSymbol + ", buy " + SubSymbol;
      if (HedgeRange > lastThreshold) sTradingOK = "Trading: OK"; else sTradingOK = "Trading: NONE";
   }
   yy = yy + yStep;   
   Write_LabelComment(iWindow, iLabelCorner, sObject, sTradingOK, "", xx1, xx1, yy);   
   yy = yy + yStep;
   Write_LabelComment(iWindow, iLabelCorner, sObject, sTrading, "", xx1, xx1, yy);   
   
   return;
}