//+------------------------------------------------------------------+
//|                                         NeutralHedge_Library.mq4 |
//|                                             2008.07.05 SwingMan |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright " 2008.07.05 SwingMan"
#property link      ""

#include <stdlib.mqh>


//####################################################################
//+------------------------------------------------------------------+
//    Set Default Pair Mirroring
//+------------------------------------------------------------------+
void Set_DefaultPairMirroring(string CurSymbol, int Pair_Mirroring, string& SubSymbol, bool& Mirroring)
{
   //-- set Mirroring ------------------------------------------------
   // "1= mirroring, 0=autom, -1= not mirror";
   if (Pair_Mirroring == 1) { 
      Mirroring = true;   
   } else 
   if (Pair_Mirroring == -1) { 
      Mirroring = false;
   }

   //-- default pairs and corelation ----------------------------------
   if (SubSymbol == "") {
      // Mirroring true
      if (CurSymbol == "GBPUSD") { SubSymbol = "EURUSD"; 
         if (Pair_Mirroring == 0) Mirroring = true; return;}   
      if (CurSymbol == "GBPJPY") {SubSymbol = "EURJPY"; 
         if (Pair_Mirroring == 0) Mirroring = true; return;} 
      // Mirroring false
      if (CurSymbol == "EURUSD") {SubSymbol = "USDCHF"; 
         if (Pair_Mirroring == 0) Mirroring = false; return;}
      if (CurSymbol == "USDJPY") {SubSymbol = "AUDUSD"; 
         if (Pair_Mirroring == 0) Mirroring = false; return;} 
   }
}

//+------------------------------------------------------------------+
//    Set Default Pair Correlation
//+------------------------------------------------------------------+
void Set_DefaultPairCorrelation(string CurSymbol, int Pair_Correlation, string& SubSymbol, bool& Correlation)
{
   //-- set Mirroring ------------------------------------------------
   // -1 = not correlated, 0= automatically, 1 =correlated
   if (Pair_Correlation == -1) { 
      Correlation = false;   
   } else 
   if (Pair_Correlation == 1) { 
      Correlation = true;
   }

   //-- default pairs and corelation ----------------------------------
   if (SubSymbol == "") {
      // Correlation false
      if (CurSymbol == "GBPUSD") { SubSymbol = "EURUSD"; 
         if (Pair_Correlation == 0) Correlation = false; return;}   
      if (CurSymbol == "GBPJPY") {SubSymbol = "EURJPY"; 
         if (Pair_Correlation == 0) Correlation = false; return;} 
      // Correlation true
      if (CurSymbol == "EURUSD") {SubSymbol = "USDCHF"; 
         if (Pair_Correlation == 0) Correlation = true; return;}
      if (CurSymbol == "USDJPY") {SubSymbol = "AUDUSD"; 
         if (Pair_Correlation == 0) Correlation = true; return;} 
   }
}


//####################################################################
//+------------------------------------------------------------------+
//    Get Threshold Value
//+------------------------------------------------------------------+
double Get_ThresholdValue(double Factor_Threshold, double RangeUpTemp[], double RangeDnTemp[], 
                          int lastBar, int firstBar, int ma_shift, int shift)
{
   double RangeCloses[1000];
   ArraySetAsSeries(RangeCloses,true);

   int Period_Statistic = firstBar - lastBar + 1;
   int i = -1;

   for (int iBar=lastBar; iBar < firstBar; iBar++) {
      i++;
      if (RangeUpTemp[iBar] != 0 && RangeUpTemp[iBar] != EMPTY_VALUE)
         RangeCloses[i] = MathAbs(RangeUpTemp[iBar]);
      else
      if (RangeDnTemp[iBar] != 0 && RangeDnTemp[iBar] != EMPTY_VALUE)
         RangeCloses[i] = MathAbs(RangeDnTemp[iBar]);
   }

   //-- take only the minim differences
   double MedRanges = iMAOnArray(    RangeCloses,0, Period_Statistic, ma_shift, MODE_SMA, shift);
   double DevRanges = iStdDevOnArray(RangeCloses,0, Period_Statistic, ma_shift, MODE_SMA, shift);
   double threshold = MedRanges + Factor_Threshold * DevRanges;
   
//Print("firstBar=",firstBar,"  MedRanges=",MedRanges,"  Factor_Threshold=",Factor_Threshold);   

   return(threshold);
} 


//####################################################################
//+------------------------------------------------------------------+
//    Write Label
//+------------------------------------------------------------------+
void Write_Label(int iWindow, int iLabelCorner, string sObject, 
                 string text1, string text2, int xx1, int xx2, int yy)
{
   int fontSize_Text = 8;
   color dColor = Orange;
   string name1 = sObject + "1" + yy;
   string name2 = sObject + "2" + yy;
   SetLabelObject(iWindow, iLabelCorner, name1, text1, fontSize_Text, dColor, xx1, yy);
   SetLabelObject(iWindow, iLabelCorner, name2, text2, fontSize_Text, dColor, xx2, yy);
   return;
}

//+------------------------------------------------------------------+
//    Write Label Comment
//+------------------------------------------------------------------+
void Write_LabelComment(int iWindow, int iLabelCorner, string sObject, 
                        string text1, string text2, int xx1, int xx2, int yy)
{
   int fontSize_Text = 8;
   color dColor = Orange;   
   string name1 = sObject + "C1" + yy;
   string name2 = sObject + "C2" + yy;
   SetLabelObject(iWindow, iLabelCorner, name1, text1, fontSize_Text, dColor, xx1, yy);
   SetLabelObject(iWindow, iLabelCorner, name2, text2, fontSize_Text, dColor, xx2, yy);
   return;
}   

//+------------------------------------------------------------------+
//    Set Label Object
//+------------------------------------------------------------------+
void SetLabelObject(int iWindow, int iLabelCorner, string sName, string sText, 
                    int fontSize_Text, color dColor, int xx, int yy)
{
   ObjectCreate(sName, OBJ_LABEL, iWindow, 0, 0);
        ObjectSetText(sName,sText,fontSize_Text, "Arial Bold", dColor);
        ObjectSet(sName, OBJPROP_CORNER, iLabelCorner);
        ObjectSet(sName, OBJPROP_XDISTANCE, xx);
        ObjectSet(sName, OBJPROP_YDISTANCE, yy);
   return;     
}

//+------------------------------------------------------------------+
//    Delete Own Objects
//+------------------------------------------------------------------+
void DeleteOwnObjects(string sObject)
{
   int i=0;
   while (i <= ObjectsTotal()) {
      if (StringFind(ObjectName(i), sObject) >= 0) ObjectDelete(ObjectName(i));
      else
      i++;
   }
   return;
}


//+------------------------------------------------------------------+
//    Send_ErrorDescription
//+------------------------------------------------------------------+
bool Get_CalculationError(string sText)
{
   int err=GetLastError();

   if (err > 0) {
      string st = Symbol() + " error(" + err + "): " + ErrorDescription(err) + " -> " + sText;
      Alert(st);
      return(false);
   } else  
      return(true);
}
//####################################################################